package com.qucheng.game.data.oldsystem.sink;

import com.qucheng.game.data.oldsystem.dao.mapper.BaseMapper;
import com.qucheng.game.data.oldsystem.pojo.MysqlConfigParam;
import com.qucheng.game.data.oldsystem.pojo.TransportMap;
import com.zaxxer.hikari.HikariConfig;
import com.zaxxer.hikari.HikariDataSource;
import org.apache.flink.api.java.tuple.Tuple3;
import org.apache.flink.configuration.Configuration;
import org.apache.flink.streaming.api.functions.sink.RichSinkFunction;
import org.apache.ibatis.mapping.Environment;
import org.apache.ibatis.session.SqlSession;
import org.apache.ibatis.session.SqlSessionFactory;
import org.apache.ibatis.session.SqlSessionFactoryBuilder;
import org.apache.ibatis.transaction.jdbc.JdbcTransactionFactory;

import javax.sql.DataSource;

public class CostSink extends RichSinkFunction<TransportMap> {
    private final MysqlConfigParam qcMysqlConfigParam;
    protected SqlSessionFactory qcSqlSessionFactory;

    public CostSink(MysqlConfigParam qcMysqlConfigParam) {
        this.qcMysqlConfigParam = qcMysqlConfigParam;
    }

    @Override
    public void open(Configuration configuration) {
        qcSqlSessionFactory = sqlSessionFactory(qcMysqlConfigParam);
    }

    @Override
    public void invoke(TransportMap value, Context context) {
        try (SqlSession session = qcSqlSessionFactory.openSession(true)) {
            BaseMapper mapper = session.getMapper(BaseMapper.class);
            mapper.saveOrUpdate(value.getTableName(), value.getPrimaryKeys(), value.getAfter());
        }
    }

    @Override
    public void close() {
    }

    private SqlSessionFactory sqlSessionFactory(MysqlConfigParam mysqlConfig) {
        // 初始化连接池
        HikariConfig config = new HikariConfig();
        config.setDriverClassName(mysqlConfig.getDriverClassName());
        config.setJdbcUrl(mysqlConfig.getUrl());
        config.setUsername(mysqlConfig.getUsername());
        config.setPassword(mysqlConfig.getPassword());
        //连接池内保留的最少连接数
        config.setMinimumIdle(1);
        DataSource dataSource = new HikariDataSource(config);

        Environment mybatisEnv = new Environment("ads-mysql", new JdbcTransactionFactory(), dataSource);

        org.apache.ibatis.session.Configuration mybatisConfig = new org.apache.ibatis.session.Configuration(mybatisEnv);
        mybatisConfig.addMapper(BaseMapper.class);
        return new SqlSessionFactoryBuilder().build(mybatisConfig);
    }
}
